<?php
/*
 * This file is part of Web Pro Manager.
 * 
 * Web Pro Manager is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Web Pro Manager is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Web Pro Manager.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @author The Web Pro Miami, Inc. <info@thewebpro.com>
 * @copyright The Web Pro Miami, Inc. <info@thewebpro.com>, http://www.thewebpro.com/
 * @license http://www.gnu.org/licenses/gpl.html GNU GPL
 * @package WebProManager
*/


// *** Upgrading from v1.17b to v1.18 ***

// config
$nDEBUG = false;
$nTmpUpgradePath = "tmp_$nUpdateVer/";


// Create new tables, if needed
$nDBDumpFile = $nTmpUpgradePath."before_install.sql"; // file that contains your SQL dump (leave blank if not needed)
if($nDEBUG){ print "*** before_install.php: Running DB dump file ($nDBDumpFile)...<BR>"; }
if(file_exists($nDBDumpFile)){
	$nDBDumpSuccess = parse_mysql_dump($nDBDumpFile);
	unlink($nDBDumpFile);
} else {
	if($nDEBUG){ print "*** STOPPING - DB dump file does NOT exist.<BR>"; }
}

// ADDITIONAL UPGRADES

$nCompPath = "../comp/";
nUpdateCompList("../comp"); // Update components list

function nUpdateCompList($nDirectory = ""){
	global $DBName,$Link,$nAllFieldARRAY,$nCompPath;
	
	$nExcludeDirARRAY = array("de","en","es","fr","img","inc","it","nav","pt","sb");
	
	// For legacy comps, set default categories
	$nCompCatARRAY = array();
	// General
	$nCompCatARRAY["basicnavbox"] = "General";
	$nCompCatARRAY["featuretext"] = "General";
	$nCompCatARRAY["headerimage"] = "General";
	$nCompCatARRAY["image_slideshow"] = "General";
	$nCompCatARRAY["imagegallery"] = "General";
	$nCompCatARRAY["pdfnavbox"] = "General";
	$nCompCatARRAY["search_results"] = "General";
	$nCompCatARRAY["simpleimage"] = "General";
	$nCompCatARRAY["simpletext"] = "General";
	$nCompCatARRAY["text"] = "General";
	$nCompCatARRAY["textimage"] = "General";
	$nCompCatARRAY["title"] = "General";
	// Forms
	$nCompCatARRAY["contactform"] = "Forms";
	$nCompCatARRAY["disqus_commentbox"] = "Forms";
	$nCompCatARRAY["loginform"] = "Forms";
	
	// Get all files in Components directory
	$nRecursive = true;
	if($nDirectory && is_dir($nDirectory)){
		$nDirHandle = @opendir($nDirectory) or die("Unable to open Component path: $nDirectory");
		for($list = array(); (FALSE !== ($nFileName = readdir($nDirHandle)));){
			if(($nFileName != '.' && $nFileName != '..') && !in_array($nFileName,$nExcludeDirARRAY) && (file_exists($path = $nDirectory.'/'.$nFileName))){
				if(is_dir($path) && ($nRecursive)){
					$list = array_merge($list, nUpdateCompList($path));
				} else {
					$entry = array('filename' => $nFileName, 'dirpath' => $nDirectory);
					
					// For all files and directories in this folder...
					do if(!is_dir($path)){
						// For all files, import if a valid component is located here
						$nCompName = strtolower(pathinfo($nFileName,PATHINFO_FILENAME));
						$nFileExt = strtolower(pathinfo($nFileName,PATHINFO_EXTENSION));
						$nFilePath = $nDirectory."/".$nFileName;
						
						// For legacy components, pull in their PHP config files and build a JSON file + comp folder if they don't already have one
						// *** Can and SHOULD be removed in a future version (after v1.04) once all old components have a JSON config file
						if(file_exists("$nFilePath") && $nFileExt=="php" && !file_exists($nDirectory."/".$nCompName.".json")){
							$nName = $nDescription = $nPageTypeLimit = $nTemplateFile = "";
							$nAllFieldARRAY = array();
							include_once("$nFilePath");
							$nCompInfoARRAY = array();
							$nCompInfoARRAY["name"] = $nName;
							$nCompInfoARRAY["version"] = "1.0";
							$nCompInfoARRAY["description"] = $nDescription;
							$nCompInfoARRAY["basefile"] = $nFileName;
							$nCompInfoARRAY["category"] = "General";
							if(isset($nCompCatARRAY[$nCompName])){ $nCompInfoARRAY["category"] = $nCompCatARRAY[$nCompName]; }
							$nCompInfoARRAY["icon"] = $nCompName.".png";
							$nCompInfoARRAY["page_type_limit"] = $nPageTypeLimit;
							$nCompInfoARRAY["template_basefile"] = $nTemplateFile;
							$nCompInfoARRAY["scripts"]["css"] = "";
							$nCompInfoARRAY["scripts"]["js"] = "";
							$nCompInfoARRAY["fields"] = $nAllFieldARRAY;
							$nCompJSON = json_encode($nCompInfoARRAY);
							$nCompJSON = nFormatJSON($nCompJSON);
							
							// Create directory for old comp and move basefile/template into it
							if(!is_dir("$nCompPath$nCompName")){
								if(mkdir("$nCompPath$nCompName",0755)){
									if(file_exists("$nCompPath$nFileName")){ rename("$nCompPath$nFileName",$nCompPath.$nCompName."/".$nFileName); }
									if(file_exists("$nCompPath$nTemplateFile")){ rename("$nCompPath$nTemplateFile",$nCompPath.$nCompName."/".$nTemplateFile); }
								} else {
									exit("Cannot write to 'comp' folder! Please update your access to the folder to allow write access (755).");
								}
							}
							
							$nFilePath = $nCompPath.$nCompName."/".$nCompName.".json";
							file_put_contents($nFilePath,$nCompJSON); // Save file
							nImportCompByJSONFile($nFilePath); // Import component, if needed
						}
						
						// Process existing JSON config files (saves or updates component, if needed)
						if(file_exists("$nFilePath") && $nFileExt=="json"){ nImportCompByJSONFile($nFilePath); }
						
						break;
					} else {
						// Actions to be performed on directories only
						break;
					} while (false);
					$list[] = $entry;
				}
			}
		}
		closedir($nDirHandle);
		return $list;
	} else {
		return false;
	}
}

// Capture calls from legacy comps and build JSON for config file
function addField($nFldID="",$nTitle="",$nCaption="",$nDefVal="",$nFldStyle="",$nCustFuncMan="",$nCustFuncSite=""){
	global $nAllFieldARRAY;
	$nAllFieldARRAY[] = array(
		"type" => $nFldID,
		"group" => "",
		"title" => $nTitle,
		"caption" => $nCaption,
		"default_value" => $nDefVal,
		"style" => $nFldStyle,
		"custom_function" => array(
			"manager" => $nCustFuncMan,
			"site" => $nCustFuncSite
		)
	);
}

// Imports JSON config file for a component and adds it to database if out-of-version or is non-existent
function nImportCompByJSONFile($nFilePath){
	global $DBName,$Link;
	if(!$nFilePath){ return; }
	if(file_exists("$nFilePath")){
		// Import file and convert to array with json_decode
		$nCompARRAY = array();
		$nCompARRAY = json_decode(file_get_contents($nFilePath),true);
		$nName = $nVersion = $nDescription = $nPageTypeLimit = $nBaseFileName = "";
		if(isset($nCompARRAY["name"])){ $nName = $nCompARRAY["name"]; }
		if(isset($nCompARRAY["version"])){ $nVersion = $nCompARRAY["version"]; }
		if(isset($nCompARRAY["description"])){ $nDescription = $nCompARRAY["description"]; }
		if(isset($nCompARRAY["page_type_limit"])){ $nPageTypeLimit = $nCompARRAY["page_type_limit"]; }
		if(isset($nCompARRAY["basefile"])){ $nBaseFileName = $nCompARRAY["basefile"]; }
		if(isset($nCompARRAY["icon"])){ $nIcon = $nCompARRAY["icon"]; }
		if(isset($nCompARRAY["category"])){ $nCategory = $nCompARRAY["category"]; }
		$nName = addslashes(stripslashes($nName));
		$nCategory = addslashes(stripslashes($nCategory));
		$nDescription = addslashes(stripslashes($nDescription));
		if($nBaseFileName && $nVersion){
			// Check if comp exists in database and if version is the same
			$nCompID = nGetCompIDFromBaseFile($nBaseFileName,$nVersion);
			if(!$nCompID){
				// If comp doesn't exist or is a different version, remove existing comp record (if any)
				$Query = "DELETE FROM nComponent WHERE nCompFile='$nBaseFileName'";
				mysql_select_db($DBName); mysql_query($Query, $Link);
				// Add new comp record
				$Query = "INSERT INTO nComponent (nName,nVersion,nDesc,nActive,nTypeLimit,nCompFile,nCategory,nIcon) VALUES ('$nName','$nVersion','$nDescription','1','$nPageTypeLimit','$nBaseFileName','$nCategory','$nIcon')";
				mysql_select_db($DBName); mysql_query($Query, $Link);
			}
		}
	}
}

function nGetCompARRAY(){
	global $DBName,$Link,$nType,$nCompPath;
	$nDataARRAY = array();
	$nSQL = "SELECT * FROM nComponent WHERE nActive=1 ORDER BY nCategory,nName";
	mysql_select_db($DBName); $Result = mysql_query($nSQL,$Link);
	while($Row = mysql_fetch_array($Result)){
		$nCompARRAY = array();
		$nName = trim($Row["nName"]);
		$nCompFile = trim($Row["nCompFile"]);
		$nCategory = trim($Row["nCategory"]);
		$nIcon = trim($Row["nIcon"]);
		$nTypeLimit = strtolower(trim($Row["nTypeLimit"]));
		$nTypeLimitARRAY = explode(",", $nTypeLimit );
		// Build data array
		if($nTypeLimit){
			// Allow base data to be returned only if component matches this page type
			if(!in_array(strtolower($nType),$nTypeLimitARRAY)){ continue; }
		}
		$nCompARRAY["nName"] = "$nName";
		$nCompARRAY["nIcon"] = "$nIcon";
		$nCompARRAY["nCompFile"] = "$nCompFile";
		$nCompARRAY["nTypeLimit"] = "$nTypeLimit";
		// Check if exists
		$nCompName = strtolower(pathinfo($nCompFile,PATHINFO_FILENAME));
		$nFilePath = $nCompPath.$nCompName."/".$nCompFile;
		if(file_exists("$nFilePath")){
			// Add to array
			$nDataARRAY["$nCategory"]["$nCompFile"] = $nCompARRAY;
		}
	}
	// Sorting (put General first if available)
	if(count($nDataARRAY)){
		if(isset($nDataARRAY["General"])) {
			$nGeneralDataARRAY = $nDataARRAY["General"];
			unset($nDataARRAY["General"]);
			$nDataARRAY = array_merge(array("General" => $nGeneralDataARRAY), $nDataARRAY);
		}
	}
	return $nDataARRAY;
}

function nGetCompIDFromBaseFile($nBaseFile,$nVersion="1.0"){
	global $DBName, $Link;
	if(!$nBaseFile){ return; }
	$nID = "";
	$Query = "SELECT nID FROM nComponent WHERE nCompFile='$nBaseFile' AND nVersion='$nVersion'";
	mysql_select_db($DBName); $Result = mysql_query($Query,$Link);
	while ($Row = mysql_fetch_array($Result)) {
		$nID = $Row["nID"];
	}
	return $nID;
}

function nGetCompIDFromName($nCompName){
	global $DBName, $Link;
	if(!$nCompName){ return; }
	$nID = "";
	$Query = "SELECT nID FROM nComponent WHERE nName='$nCompName'";
	mysql_select_db($DBName); $Result = mysql_query($Query,$Link);
	while ($Row = mysql_fetch_array($Result)) {
		$nID = $Row["nID"];
	}
	return $nID;
}

function nFormatJSON($json){
    $result = '';
    $level = 0;
    $prev_char = '';
    $in_quotes = false;
    $ends_line_level = NULL;
    $json_length = strlen( $json );
    for( $i = 0; $i < $json_length; $i++ ) {
        $char = $json[$i];
        $new_line_level = NULL;
        $post = "";
        if( $ends_line_level !== NULL ) {
            $new_line_level = $ends_line_level;
            $ends_line_level = NULL;
        }
        if( $char === '"' && $prev_char != '\\' ) {
            $in_quotes = !$in_quotes;
        } else if( ! $in_quotes ) {
            switch( $char ) {
                case '}': case ']':
                    $level--;
                    $ends_line_level = NULL;
                    $new_line_level = $level;
                    break;
                case '{': case '[':
                    $level++;
                case ',':
                    $ends_line_level = $level;
                    break;
                case ':':
                    $post = " ";
                    break;
                case " ": case "\t": case "\n": case "\r":
                    $char = "";
                    $ends_line_level = $new_line_level;
                    $new_line_level = NULL;
                    break;
            }
        }
        if( $new_line_level !== NULL ) {
            $result .= "\n".str_repeat( "\t", $new_line_level );
        }
        $result .= $char.$post;
        $prev_char = $char;
    }
    return $result;
}